-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftClassData")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------------

local playerClasses = { }

-----------------------------------------------------------------------------

local AloftClassData = Aloft:NewModule("ClassData", "AceEvent-2.0")
AloftClassData.db = Aloft:AcquireDBNamespace("classData")
Aloft:RegisterDefaults("classData", "server", {
	playerClasses = { }
})

-----------------------------------------------------------------------------

function AloftClassData:OnEnable()
	playerClasses = self.db.server.playerClasses

	self:RegisterEvent("Aloft:OnNameplateUpdateData", "OnNameplateUpdateData")

	-- Events to update our class database
    self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "UpdateClassFromMouseOver")
	self:RegisterEvent("GUILD_ROSTER_UPDATE", "UpdateClassesFromGuild")
    self:RegisterEvent("WHO_LIST_UPDATE", "UpdateClassesFromWho") 
    self:RegisterEvent("CHAT_MSG_SYSTEM", "UpdateClassesFromWho") 
    self:RegisterEvent("RosterLib_RosterUpdated", "UpdateClassesFromRoster")

	self:RegisterEvent("FRIENDLIST_UPDATE", "UpdateClassesFromFriends")

	-- Update stuff from battlegrounds
	self:ScheduleRepeatingEvent(function() if MiniMapBattlefieldFrame.status == "active" then RequestBattlefieldScoreData() end end, 15)
	self:RegisterEvent("UPDATE_BATTLEFIELD_SCORE", "UpdateClassesFromBattlefield")
	
	self:UpdateClassesFromGuild()
	self:UpdateClassesFromFriends()
end

function AloftClassData:OnDisable()
	self:UnregisterAllEvents()
end

-----------------------------------------------------------------------------

function AloftClassData:OnNameplateUpdateData(aloftData)
	aloftData.class = nil
	
	-- This is so that pets named the same as players we've encountered aren't given the wrong data
	if aloftData.type == "friendlyPlayer" or aloftData.type == "hostile" then
		aloftData.class = playerClasses[aloftData.name]
	end
end

-----------------------------------------------------------------------------

local updateRequired = false

local function AddData(name, class)
	if playerClasses[name] ~= class then
		playerClasses[name] = class
		updateRequired = true
	end
end

function AloftClassData:UpdateVisibleNameplates()
	if updateRequired then
		for frame in Aloft:IterateVisibleNameplates() do
			local aloftData = frame.aloftData
			local class = playerClasses[aloftData.name]
		
			if aloftData.class ~= class then
				aloftData.class = class
				self:TriggerEvent("Aloft:OnClassDataChanged", frame)
			end
		end
	end
	updateRequired = false
end

-----------------------------------------------------------------------------

function AloftClassData:UpdateClassFromMouseOver()
	if UnitIsPlayer("mouseover") then
		AddData(UnitName("mouseover"), select(2, UnitClass("mouseover")))
		self:UpdateVisibleNameplates()
	end
end

function AloftClassData:UpdateClassesFromGuild()
    if IsInGuild() then
		for i = 1, GetNumGuildMembers(true) do
			local name, _, _, _, class = GetGuildRosterInfo(i)
			AddData(name, string.upper(class))
		end
		self:UpdateVisibleNameplates()
	end
end

function AloftClassData:UpdateClassesFromWho()
    for i = 1, GetNumWhoResults() do
        local name, _, _, _, class = GetWhoInfo(i)
        AddData(name, string.upper(class))
    end
	self:UpdateVisibleNameplates()
end

function AloftClassData:UpdateClassesFromRoster()
	for unit in roster:IterateRoster(false) do
		AddData(unit.name, unit.class)
	end
	self:UpdateVisibleNameplates()
end

function AloftClassData:UpdateClassesFromBattlefield()
	for i = 1, GetNumBattlefieldScores() do
		local name, _, _, _, _, _, _, _, _, class = GetBattlefieldScore(i)
		name = string.gsub(name, "(%w+)-.+", "%1")
		
		AddData(name, class)
	end
	self:UpdateVisibleNameplates()
end

function AloftClassData:UpdateClassesFromFriends()
    for i = 1, GetNumFriends() do
        local name, _, class = GetFriendInfo(i)
		class = string.upper(class)

		if class ~= "UNKNOWN" then
			AddData(name, class)
		end
    end
	self:UpdateVisibleNameplates()
end

-----------------------------------------------------------------------------
