-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftNameText")
local SML = AceLibrary("SharedMedia-1.0")
local SML_font = SML:List("font")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Name Text"] = true,
	["Nameplate text options"] = true,
	["Enable"] = true,
	["Show unit names"] = true,
	["Font"] = true,
	["Sets the font for nameplate text"] = true,
	["Font Shadow"] = true,
	["Show font shadow on nameplate text"] = true,
	["Font Size"] = true,
	["Sets the font height of the nameplate text"] = true,
	["Outline"] = true,
	["Sets the font outline for nameplate text"] = true,
	["Alignment"] = true,
	["Sets the alignment of the nameplate text"] = true,	
	["Position"] = true,
	["Adjust nameplate text position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the text"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the text"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the text"] = true,
	
	["Override Colors"] = true,
	["Override the games colors for nameplate text"] = true,
	["Colors"] = true,
	["Customize colors"] = true,
	["Friendly Players"] = true,
	["Sets the nameplate color for friendly players"] = true,
	["Friendly NPCs"] = true,
	["Sets the nameplate bar color for friendly NPCs"] = true,
	["Friendly Boss"] = true,
	["Sets the nameplate color for friendly NPC Bosses"] = true,
	["Hostile Units"] = true,
	["Sets the nameplate color for hostile units"] = true,
	["Hostile Boss"] = true,
	["Sets the nameplate color for hostile boss units"] = true,
	["Neutral Units"] = true,
	["Sets the nameplate color for neutral units"] = true,
	["Group Pets"] = true,
	["Sets the nameplate color for group pets"] = true,

	["None"] = true,
	["Normal"] = true,
	["Thick"] = true,
} end)

-----------------------------------------------------------------------------

local AloftNameText = Aloft:NewModule("NameText", "AceEvent-2.0")
AloftNameText.db = Aloft:AcquireDBNamespace("nameText")
Aloft:RegisterDefaults("nameText", "profile", {
	enable			= true,
	font			= "Arial Narrow",
	fontSize		= 10,
	shadow			= false,
	alignment		= "LEFT",
	outline			= "OUTLINE",
	offsets =
	{
		left		= 0,
		right		= 0,
		vertical	= 0,
	},
	overrideColors	= true,
	colors =
	{
		friendlyPlayer	= { 1, 1, 1 },
		friendlyNPC		= { 1, 1, 1 },
		friendlyBoss	= { 1, 1, 1 },
		hostile			= { 1, 1, 1 },
		hostileBoss		= { 1, 1, 1 },
		neutral			= { 1, 1, 1 },
		groupPet		= { 1, 1, 1 },
	},
})

Aloft.Options.args.nameText =
{
	type = 'group',
	name = L["Name Text"],
	desc = L["Nameplate text options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show unit names"],
			get = function() return AloftNameText.db.profile.enable end,
			set = function(v) AloftNameText.db.profile.enable = v AloftNameText:UpdateAll() end,
			order = 1
		},
		font =
		{
			type = "text", 
			name = L["Font"], 
			desc = L["Sets the font for nameplate text"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			get = function() return AloftNameText.db.profile.font end,
			set = function(v) AloftNameText.db.profile.font = v AloftNameText:UpdateAll() end,
			validate = SML_font,
		},
		fontSize =
		{
			type = 'range',
			name = L["Font Size"],
			desc = L["Sets the font height of the nameplate text"],
			max = 16,
			min = 4,
			step = 1,
			disabled = function() return not AloftNameText.db.profile.enable end,
			get = function() return AloftNameText.db.profile.fontSize end,
			set = function(value) AloftNameText.db.profile.fontSize = value AloftNameText:UpdateAll() end
		},
		shadow =
		{
			type = 'toggle',
			name = L["Font Shadow"],
			desc = L["Show font shadow on nameplate text"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			get = function() return AloftNameText.db.profile.shadow end,
			set = function(v) AloftNameText.db.profile.shadow = v AloftNameText:UpdateAll() end
		},
		alignment =
		{
			type = 'text',
			name = L["Alignment"],
			desc = L["Sets the alignment of the nameplate text"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			get = function() return AloftNameText.db.profile.alignment end,
			set = function(value) AloftNameText.db.profile.alignment = value AloftNameText:UpdateAll() end,
			validate = {"LEFT", "CENTER", "RIGHT" },
		},
		outline =
		{
			type = 'text',
			name = L["Outline"],
			desc = L["Sets the font outline for nameplate text"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			get = function() return AloftNameText.db.profile.outline end,
			set = function(value) AloftNameText.db.profile.outline = value AloftNameText:UpdateAll() end,
			validate = { [""] = L["None"], ["OUTLINE"] = L["Normal"], ["THICKOUTLINE"] = L["Thick"] },
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust nameplate text position"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the text"],
					min = -100,
					max = 100,
					step = 1,
					get = function() return AloftNameText.db.profile.offsets.left end,
					set = function(v) AloftNameText.db.profile.offsets.left = v AloftNameText:UpdateAll() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the text"],
					min = -100,
					max = 100,
					step = 1,
					get = function() return AloftNameText.db.profile.offsets.right end,
					set = function(v) AloftNameText.db.profile.offsets.right = v AloftNameText:UpdateAll() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the text"],
					min = -64,
					max = 64,
					step = 1,
					get = function() return AloftNameText.db.profile.offsets.vertical end,
					set = function(v) AloftNameText.db.profile.offsets.vertical = v AloftNameText:UpdateAll() end
				},
			},
		},
		overrideColors =
		{
			type = 'toggle',
			name = L["Override Colors"],
			desc = L["Override the games colors for nameplate text"],
			disabled = function() return not AloftNameText.db.profile.enable end,
			get = function() return AloftNameText.db.profile.overrideColors end,
			set = function(value) AloftNameText.db.profile.overrideColors = value AloftNameText:UpdateAll() end,
		},
		colors =
		{
			type = 'group',
			name = L["Colors"],
			desc = L["Customize colors"],
			disabled = function() return not AloftNameText.db.profile.enable or not AloftNameText.db.profile.overrideColors end,
			args =
			{
				friendlyPlayer =
				{
					type = 'color',
					name = L["Friendly Players"],
					desc = L["Sets the nameplate color for friendly players"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyPlayer) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.friendlyPlayer = { r, g, b } AloftNameText:UpdateAll() end,
				},
				friendlyNPC =
				{
					type = 'color',
					name = L["Friendly NPCs"],
					desc = L["Sets the nameplate bar color for friendly NPCs"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyNPC) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.friendlyNPC = { r, g, b } AloftNameText:UpdateAll() end,
				},
				friendlyBoss =
				{
					type = 'color',
					name = L["Friendly Boss"],
					desc = L["Sets the nameplate color for friendly NPC Bosses"],
					get = function() return unpack(AloftNameText.db.profile.colors.friendlyBoss) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.friendlyBoss = { r, g, b } AloftNameText:UpdateAll() end,
				},
				hostile =
				{
					type = 'color',
					name = L["Hostile Units"],
					desc = L["Sets the nameplate color for hostile units"],
					get = function() return unpack(AloftNameText.db.profile.colors.hostile) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.hostile = { r, g, b } AloftNameText:UpdateAll() end,
				},
				hostileBoss =
				{
					type = 'color',
					name = L["Hostile Boss"],
					desc = L["Sets the nameplate color for hostile boss units"],
					get = function() return unpack(AloftNameText.db.profile.colors.hostileBoss) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.hostileBoss = { r, g, b } AloftNameText:UpdateAll() end,
				},
				neutral =
				{
					type = 'color',
					name = L["Neutral Units"],
					desc = L["Sets the nameplate color for neutral units"],
					get = function() return unpack(AloftNameText.db.profile.colors.neutral) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.neutral = { r, g, b } AloftNameText:UpdateAll() end,
				},
				groupPet =
				{
					type = 'color',
					name = L["Group Pets"],
					desc = L["Sets the nameplate color for group pets"],
					get = function() return unpack(AloftNameText.db.profile.colors.groupPet) end,
					set = function(r, g, b) AloftNameText.db.profile.colors.groupPet = { r, g, b } AloftNameText:UpdateAll() end,
				},
			}
		}
	},
}

-----------------------------------------------------------------------------

function AloftNameText:UpdateAll()
	for frame in Aloft:IterateNameplates() do
		self:SetupFrame(frame)
	end

	if self.db.profile.enable and self.db.profile.overrideColors then
		self:RegisterEvent("Aloft:OnNameplateShow", "UpdateText")
		self:RegisterEvent("Aloft:OnHealthBarColorChanged", "UpdateText")

		for frame in Aloft:IterateVisibleNameplates() do
			self:UpdateText(frame)
		end
	else
		self:CancelEvent("Aloft:OnNameplateShow")
		self:CancelEvent("Aloft:OnHealthBarColorChanged")
	end
end

-----------------------------------------------------------------------------

function AloftNameText:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")

	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)
	
	self:UpdateAll()
end

function AloftNameText:OnDisable()
	self:UnregisterAllEvents()

	for frame in Aloft:IterateNameplates() do
		local nameTextFrame = frame.aloftData.nameTextFrame

		nameTextFrame:ClearAllPoints()
		nameTextFrame:SetPoint("BOTTOM", frame, "CENTER")
		nameTextFrame:SetFont(NAMEPLATE_FONT, 14.5)		
		nameTextFrame:SetText(frame.aloftData.name)
		nameTextFrame:SetShadowOffset(1, -1)
		nameTextFrame:Show()
	end
end

function AloftNameText:SetupFrame(frame)
	local nameTextFrame = select(5, frame:GetRegions())
	local healthBar = frame.aloftData.healthBar

	nameTextFrame:ClearAllPoints()
	nameTextFrame:SetPoint("TOPLEFT", healthBar, "TOPLEFT", self.db.profile.offsets.left, self.db.profile.offsets.vertical)
	nameTextFrame:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", self.db.profile.offsets.right, self.db.profile.offsets.vertical)
	self:PrepareText(nameTextFrame, self.db.profile)

	if self.db.profile.enable then
		nameTextFrame:Show()
	else
		nameTextFrame:Hide()
	end

	frame.aloftData.nameTextFrame = nameTextFrame
end

function AloftNameText:UpdateText(frame)
	local color = self.db.profile.colors[frame.aloftData.type]

	if color then
		frame.aloftData.nameTextFrame:SetText(string.format("|cff%02x%02x%02x%s", color[1]*255, color[2]*255, color[3]*255, frame.aloftData.name))
	end
end

-----------------------------------------------------------------------------
